/***************************************************************************
                          audio_thread.cpp  -  description
                             -------------------
    begin                : 2002
    copyright            : (C) 2002-2005 by Philip McLeod
    email                : pmcleod@cs.otago.ac.nz
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   Please read LICENSE.txt for details.
 ***************************************************************************/
#include <stdio.h>
#include <math.h>
#include <qrect.h>
//Added by qt3to4:
#include <QCustomEvent>
#include <QMutex>

#include "audio_thread.h"
#include "audio_stream.h"
#include "sound_file_stream.h"
#include "freqpair.h"
#include "gdata.h"
#include "Filter.h"
#include "bspline.h"
#include "soundfile.h"
#include "channel.h"

#include <QMessageBox>
#include "main.h"

AudioThread::AudioThread() {
  //printf("Warning - audio thread created with no sound file specified.\n");
  _playSoundFile = NULL;
  _recSoundFile = NULL;
  sleepCount = 0;
}

#ifdef SRT
void AudioThread::start() {
  fprintf(stderr, "Warning - audio thread created with no sound file specified.\n");
  _playSoundFile = NULL;
  _recSoundFile = NULL;
  stopping = false;
  QMessageBox::warning(mainWindow, "Error", QString("Starting thread."), QMessageBox::Ok, QMessageBox::NoButton);
  QThread::start(QThread::HighPriority/*TimeCriticalPriority*/);
}
#endif

void AudioThread::start(SoundFile *sPlay, SoundFile *sRec) {
  fprintf(stderr, "Audio thread created for\n");
  if(sPlay)
    fprintf(stderr, "Playing file %s\n", sPlay->filename); fflush(stdout);
  if(sRec)
    fprintf(stderr, "Recording file %s\n", sRec->filename); fflush(stdout);
  _playSoundFile = sPlay;
  _recSoundFile = sRec;
  stopping = false;
  QThread::start(QThread::HighPriority/*TimeCriticalPriority*/);
}

/** Causes the audio thread to stop at the end of the next loop
*/
void AudioThread::stop()
{
  stopping = true;
}

/** Stop the audio thread and waits for it to finish
*/
void AudioThread::stopAndWait()
{
  stop();
  wait();
}

void AudioThread::run()
{
  first = true;
  fast_update_count = 0;
  slow_update_count = 0;
  frame_num = 0;

  //read to the 1 chunk befor time 0
  if((gdata->soundMode & SOUND_REC)) {
    gdata->setDoingActiveAnalysis(true);
    myassert(_recSoundFile->firstTimeThrough == true);
    _recSoundFile->recordChunk(_recSoundFile->offset());
  }
  
  gdata->running = STREAM_FORWARD;

  while(!stopping) {
    if(doStuff() == 0) break;
#ifdef WINDOWS
    Sleep(0); //make other threads do some stuff
#else
    sleep(0); //make other threads do some stuff
#endif
  }

  gdata->running = STREAM_STOP;

  if((gdata->soundMode & SOUND_REC)) {
    gdata->setDoingActiveAnalysis(false);
    _recSoundFile->firstTimeThrough = false;
    //
    // SRT -- Switches over to play when fisnished recording?
    //
#ifdef SRT    
    _recSoundFile->rec2play();
    gdata->soundMode = SOUND_PLAY;
#endif
  }

  if(gdata->audio_stream) {
    delete gdata->audio_stream;
    gdata->audio_stream = NULL;
  }
  _playSoundFile = NULL;
  _recSoundFile = NULL;
  
}

int AudioThread::doStuff()
{
  int force_update = false;
  if(gdata->running == STREAM_PAUSE) { msleep(20); return 1; } //paused
  if(!_playSoundFile && !_recSoundFile) return 0;

  ++frame_num;

  if(gdata->running == STREAM_UPDATE) {
    gdata->running = STREAM_PAUSE; //update then pause
    force_update = true;
  }
  if(gdata->running != STREAM_FORWARD) return 0;
  
#ifdef SRT
  //  This is the main block of code for reading or write the next
  //  chunk to the soundcard or file
  if(gdata->soundMode == SOUND_PLAY) {
    if(!_playSoundFile->playChunk()) { //end of file
      return 0; //stop the audio thread playing
    }
    if(!gdata->audio_stream) {
      if (++sleepCount % 4 == 0) {
        int sleepval = (1000 * _playSoundFile->framesPerChunk()) / _playSoundFile->rate();
        msleep(sleepval * 4);
      }
    }
  } else
#endif
    
    if(gdata->soundMode == SOUND_REC) {  // SOUND_REC
    int bufferChunks = gdata->audio_stream->inTotalBufferFrames() / _recSoundFile->framesPerChunk();
    if(frame_num > bufferChunks)
      _recSoundFile->recordChunk(_recSoundFile->framesPerChunk());
    }
  
#ifdef SRT  
  else if(gdata->soundMode == SOUND_PLAY_REC) {
    int bufferChunks = gdata->audio_stream->inTotalBufferFrames() / _recSoundFile->framesPerChunk();
    if(frame_num > bufferChunks) {
      if(!SoundFile::playRecordChunk(_playSoundFile, _recSoundFile)) {
	//end of file
        return 0; //stop the audio thread playing
      }
    }
  }
#endif  

  //Set some flags to cause an update of views, every now and then
  //
  // SRT -- this was all ignored anyway...
  //
#ifdef SRT  
  fast_update_count++;
  slow_update_count++;
  int slowUpdateAfter = toInt(double(gdata->slowUpdateSpeed()) / 1000.0 / curSoundFile()->timePerChunk());
  int fastUpdateAfter = toInt(double(gdata->fastUpdateSpeed()) / 1000.0 / curSoundFile()->timePerChunk());
  if(!gdata->need_update) {
    if((slow_update_count >= slowUpdateAfter) || force_update) {
      gdata->need_update = true;
      fast_update_count = 0;
      slow_update_count = 0;
    } else if(fast_update_count >= fastUpdateAfter) {
      gdata->need_update = true;
      fast_update_count = 0;
    }
  }
#endif 

  gdata->view->doUpdate();

  return 1;
}
