//
//  Thu Jun 26 21:48:31 2008 -- Scott R. Turner
//  BARVIEW
#include <qpixmap.h>
#include <qpainter.h>
//Added by qt3to4:
#include <QPaintEvent>
#include <Qt>
#include "barwidget.h"
#include "gdata.h"
#include "channel.h"
#include "analysisdata.h"
#include "useful.h"

BarWidget::BarWidget(QWidget *parent)
  : DrawWidget(parent)
{
  textY = 0;
  textX = 0;
  //make any connections
  connect(gdata->view, SIGNAL(onSlowUpdate(double)), this, SLOT(update()));
}

void BarWidget::printString(const QString &s) {
  p.setBackgroundMode(Qt::TransparentMode);
  p.setPen("black");
  textY += fontHeight;
  p.drawText(textX, textY, s);
}

void BarWidget::printString(const QString &s, const char *col) {
  p.setBackgroundMode(Qt::OpaqueMode);
  p.setBackground(QBrush(col));
  textY += fontHeight;
  p.drawText(textX, textY, s);
}

void BarWidget::paintEvent( QPaintEvent * )
{
  RTTAinfo *rInfo = gdata->rtta_data->rttaInfo;
  int total_samples = gdata->rtta_data->total_samples;
  QFont font("Courier", 10, QFont::Bold);
  QString s;

  beginDrawing();

  // Drawing code goes here
  p.setFont(font);
  QFontMetrics fm = p.fontMetrics();
  fontHeight = fm.height() + 5;

  // Get the size of the widget
  QSize qsize;
  int height, width;

  qsize = size();
  height = qsize.height();
  width = qsize.width();

  //
  //  Labels
  //
  p.drawText(5,fontHeight,QString("Flat"));
  p.drawText(width-45,fontHeight,QString("Sharp"));

  //
  //  Draw bars
  //
  if (total_samples > 0) {
    int cutoff = (int) total_samples * gdata->getSamplesThreshold();
    int cents;
    int yellow_limit = gdata->getYellowLimit();
    int red_limit = gdata->getRedLimit();
    char *color = "green";
    //
    //  Unlike text, filled rectangle starts at the upper left, not
    //  the lower left.
    //
    int ypos = fontHeight + 12;
    int xpos = width/2;
    int bar_height = fontHeight-10;
    int bar_length=0;
    int max_bar_length = (width/2)-5;

    for(int i=0; i<RTTA_NOTE_RANGE; i++) {
      if (rInfo[i].num_samples > cutoff) {
	//
	//  Cents is +/- 50
	//
	cents = (int) (((rInfo[i].total/rInfo[i].num_samples) -
			rInfo[i].start_val - 0.50) * 100);
	//
	//  Red and yellow limits
	//
	if (cents > red_limit || cents < -red_limit) {
	  color = "red";
	} else if (cents > yellow_limit || cents < -yellow_limit) {
	  color = "yellow";
	} else {
	  color = "green";
	};
	//
	//  Figure this bar length
	//
	//	bar_length = (int) (((float) cents) / 50.0) * (float) max_bar_length;
	bar_length = (int) (((float) cents * (float) max_bar_length) / 50.0);
	if (bar_length < 0) {
	  p.fillRect(xpos+bar_length,ypos,-bar_length,bar_height,QBrush(QColor(color)));
	  p.drawRect(xpos+bar_length,ypos,-bar_length,bar_height);
	} else {
	  p.fillRect(xpos,ypos,bar_length,bar_height,QBrush(QColor(color)));
	  p.drawRect(xpos,ypos,bar_length,bar_height);
	};
	//
	// Down to next bar
	//
	// p.drawText(xpos,ypos,1000,1000, Qt::AlignLeft, QString("*"));
	ypos += fontHeight;
      }
    }
    //
    //  Base line
    //
    p.drawLine(width/2,fontHeight,width/2,ypos);
  }


  endDrawing();
}
