/***************************************************************************
                          audio_stream.h  -  windows audio routines
                             -------------------
    begin                : 2003
    copyright            : (C) 2003-2005 by Philip McLeod
    email                : pmcleod@cs.otago.ac.nz
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   Please read LICENSE.txt for details.
 ***************************************************************************/
#ifndef AUDIOSTREAM_H
#define AUDIOSTREAM_H

#include <stdio.h>
#include "sound_stream.h"
#include <windows.h>
#include <Mmsystem.h>

#include <qthread.h>

#include <map>

struct AudioBuffer {
  WAVEHDR *wavehdr;
  int cur_buf;
  int num_bufs;
  int buf_pos;
  int buffer_size;
  volatile int free_bufs;
  QMutex bufs_mutex;
  HANDLE bufferEvent;

  AudioBuffer() { buffer_size = 1024; }
  void init(int buffer_size_, int num_bufs_, int free_bufs_) {
	  buffer_size = buffer_size_;
	  cur_buf = 0;
	  buf_pos = 0;
	  num_bufs = num_bufs_;
	  free_bufs = free_bufs_;
  }
  int wait_for_buffer();
};

class AudioStream : public SoundStream
{

  static std::map<QString, uint> inputDevices;
  static std::map<QString, uint> outputDevices;

 public:
  uint in_device_num;
  uint out_device_num;
  //int buffer_size;
  int pcm_vol;

  HWAVEOUT hwo;
  HWAVEIN hwi;
  AudioBuffer in, out;
  /*  WAVEHDR *wavehdr;
  int cur_buf;
  int num_bufs;
  int buf_pos;
  volatile int free_bufs;
  QMutex bufs_mutex;
  HANDLE bufferEvent;
*/  bool been_init;

  AudioStream();
  virtual ~AudioStream();

  int inBufferSize() { return in.buffer_size; }
  int outBufferSize() { return out.buffer_size; }
  int inNumBuffers() { return in.num_bufs; }
  int outNumBuffers() { return out.num_bufs; }
  int inTotalBufferFrames() { return in.buffer_size * in.num_bufs; }
  int outTotalBufferFrames() { return out.buffer_size * out.num_bufs; }

  void close();

  void signal1();
  void print_state();
  void set_params();
  int open(int mode_, int freq_=44100, int channels_=2, int bits_=16, int buffer_size_=2048);
  long read_bytes(void *data, long length);
  long read_frames(void *data, long length);

  long write_bytes(void *data, long length);
  long write_frames(void *data, long length);
  long wait_bytes(long length);
  long wait_frames(long length);

  static QStringList getOutputDeviceNames();
  static QStringList getInputDeviceNames();

  static uint getInputDeviceNumber(QString toFind);
  static uint getOutputDeviceNumber(QString toFind);

  
};


#endif
